@@ -20,7 +20,9 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #----------------------------------------------------------------------------
-#import dbus.glib
+modulesFolder = 'modules'
+import sys
+sys.path.append(modulesFolder)
 import time
 startTimestamp = time.time()
 import pygtk
@@ -28,14 +30,38 @@ pygtk.require('2.0')
 import gobject
 import gtk
 import math
-import sys
 import traceback
 import os
 from gtk import gdk
 from math import radians
+from configobj import ConfigObj
 importsDoneTimestamp = time.time()
 
 
+def createFolderPath(newPath):
+  """
+  Creat a path for a directory and all needed parent forlders
+  -> parent directoryies will be created
+  -> if directory already exists, then do nothing
+  -> if there is another filsystem object (like a file) with the same name, raise an exception
+  """
+  if not newPath:
+    print("cannot create folder, wrong path: ", newPath)
+    return False
+  if os.path.isdir(newPath):
+    return True
+  elif os.path.isfile(newPath):
+    print("cannot create directory, file already exists: '%s'" % newPath)
+    return False
+  else:
+    print("creating path: %s" % newPath)
+    head, tail = os.path.split(newPath)
+    if head and not os.path.isdir(head):
+        mkdirs(head)
+    if tail:
+        os.mkdir(newPath)
+    return True
+
 def update1(mapWidget):
   mapWidget.update()
   return(True)
@@ -67,14 +93,12 @@ class MapWidget(gtk.Widget):
 
     self.msLongPress = 400
 
-    self.modulesFolder = 'modules'
 
     """ setting this both to 100 and 1 in mapView and gpsd fixes the arow orientation bug """
     self.timer1 = gobject.timeout_add(100, update1, self) #default 100
-#    self.timer2 = gobject.timeout_add(10, update2, self) #default 10
     self.timer3 = None # will be used for timing long press events
-    self.d = {} # List of data
-    self.m = {} # List of modules
+    self.d = {} # persistant dictionary of data
+    self.m = {} # dictionary of loaded modules
     self.watches = {} # List of data change watches
     self.maxWatchId = 0
     self.watch('needRedraw', self._checkForRedrawCB) # react on redraw requests
@@ -102,12 +126,19 @@ class MapWidget(gtk.Widget):
     self.lastFullRedraw = time.time()
     self.lastFullRedrawRequest = time.time()
 
+    # map layers
+    self.mapLayers = {}
+    self.notificationModule = None
+
+    # per mode options
+    # NOTE: this variable is automatically saved by the
+    # options module
+    self.keyModifiers = {}
+
   def loadModules(self):
     """Load all modules from the specified directory"""
 
-    modulesFolder = self.modulesFolder
     device = self.device
-    sys.path.append(modulesFolder)
     print "importing modules:"
     start = time.clock()
     self.initInfo={
@@ -155,6 +186,9 @@ class MapWidget(gtk.Widget):
     # run what needs to be done after firstTime is called
     self._modulesLoadedPostFirstTime()
 
+    # get the notication module
+    self.notificationModule = self.m.get('notification', None)
+
     print "Initialization complete in %1.2f ms" % (1000 * (time.clock() - start))
 
   def _getModuleNamesFromFolder(self,folder):
@@ -198,6 +232,10 @@ class MapWidget(gtk.Widget):
     self.watch('viewport', self._updateCenteringShiftCB)
     # and map scaling
     self.watch('mapScale', self._updateCenteringShiftCB)
+    # and mode change
+    self.watch('mode', self._modeChangedCB)
+    # cache key modifiers
+    self.keyModifiers = self.d.get('keyModifiers', {})
 
   def _modulesLoadedPostFirstTime(self):
     """this is run after all the modules have been loaded,
@@ -216,13 +254,49 @@ class MapWidget(gtk.Widget):
     
   ## OPTIONS SETTING AND WATCHING ##
 
-  def get(self, name, default=None):
+  def get(self, name, default=None, mode=None):
     """Get an item of data"""
-    return(self.d.get(name, default))
 
-  def set(self, name, value, save=False):
-    """Set an item of data"""
-    self.d[name] = value
+    # check if the value depends on current mode
+    if name in self.keyModifiers.keys():
+      # get the current mode
+      if mode == None:
+        mode = self.d.get('mode', 'car')
+      if mode in self.keyModifiers[name]['modes'].keys():
+        # get the dictionary with per mode values
+        multiDict = self.d.get('%s#multi' % name , {})
+        # retrun the value for current mode
+        return multiDict.get(mode,default)
+      else:
+        return(self.d.get(name, default))
+
+    else: # just return the normal value
+      return(self.d.get(name, default))
+
+  def set(self, name, value, save=False, mode=None):
+    """Set an item of data,
+    if there is a watch set for this key,
+    notify the watcher that its value has changed"""
+
+    oldValue = self.get(name, value)
+
+    if name in self.keyModifiers.keys():
+      # get the current mode
+      if mode == None:
+        mode = self.d.get('mode', 'car')
+      # check if there is a modifier for the current mode
+      if mode in self.keyModifiers[name]['modes'].keys():
+        # save it to the name + #multi key under the mode key
+        try:
+          self.d['%s#multi' % name][mode] = value
+        except KeyError: # key not yet created
+          self.d['%s#multi' % name] = {mode : value}
+      else: # just save to the key as usuall
+        self.d[name] = value
+    else: # just save to the key as usuall
+      self.d[name] = value
+
+    self._notifyWatcher(name, oldValue)
     """options are normally saved on shutdown,
     but for some data we want to make sure they are stored and not
     los for example becuase of power outage/empty battery, etc."""
@@ -230,12 +304,33 @@ class MapWidget(gtk.Widget):
       options = self.m.get('options')
       if options:
         options.save()
-    if name in self.watches.keys():
-      self._notifyWatcher(name, value)
+
+  def optionsKeyExists(self, key):
+    """Report if a given key exists"""
+    return key in self.d.keys()
+
+  def purgeKey(self, key):
+    """remove a key from the presistant dictionary,
+    including possible key modifiers and alternate values"""
+    if key in self.d:
+      oldValue = self.get(key, None)
+      del self.d[key]
+      # purge any key modifiers
+      if key in self.keyModifiers.keys():
+        del self.keyModifiers[key]
+        """also remove the possibly present
+        alternative states for different modes"""
+        multiKey = "%s#multi" % key
+        if multiKey in self.d:
+          del self.d[multiKey]
+      self._notifyWatcher(key, oldValue)
+      return True
+    else:
+      print("modrana: can't purge a not present key: %s" % key)
 
   def watch(self, key, callback, *args):
     """add a callback on an options key
-    callbakc will get:
+    callback will get:
     key, newValue, oldValue, *args
 
     NOTE: watch ids should be >0, so that they evaluate as True
@@ -253,23 +348,137 @@ class MapWidget(gtk.Widget):
     (nrId, key) = id.split('_')
 
     if key in self.watches:
-      remove = lambda x:x==id
+      remove = lambda x:x[0]==id
       self.watches[key][:] = [x for x in self.watches[key] if not remove(x)]
     else:
-      print "modRana: cant remove watch - key does not exist, watchId:", id
+      print("modRana: can't remove watch - key does not exist, watchId:", id)
 
-  def _notifyWatcher(self, key, newValue):
-    """run callbacks registered on an options key"""
+  def _notifyWatcher(self, key, oldValue):
+    """run callbacks registered on an options key
+    HOW IT WORKS
+    * the watcher is notified before the key is written to the persistant
+    dictionary, so that it can react before the change is visible
+    * the watcher gets the key and both the new and old values
+    """
     callbacks = self.watches.get(key, None)
     if callbacks:
       for item in callbacks:
         (id,callback,args) = item
-        oldValue = self.get(key, None)
+        # rather supply the old value than None
+        newValue = self.get(key, oldValue)
         if callback:
-          callback(key,newValue,oldValue, *args)
+          if callback(key,oldValue, newValue, *args) == False:
+            # remove watches that return False
+            self.removeWatch(id)
         else:
           print "invalid watcher callback :", callback
 
+  def addKeyModifier(self, key, modifier=None, mode=None, copyInitialValue=True):
+    """add a key modifier
+    NOTE: currently only used to make value of some keys
+    dependent on the current mode"""
+    options = self.m.get('options', None)
+    # remeber the old value, if not se use default from options
+    # if available
+    if options:
+      defaultValue = options.getKeyDefault(key, None)
+    else:
+      defaultValue = None
+    oldValue = self.get(key,defaultValue)
+    if mode == None:
+      mode = self.d.get('mode', 'car')
+    if key not in self.keyModifiers.keys(): # initialize
+      self.keyModifiers[key] = {'modes':{mode:modifier}}
+    else:
+      self.keyModifiers[key]['modes'][mode] = modifier
+
+    # make sure the multi mode dictionary exists
+    multiKey = '%s#multi' % key
+    multiDict = self.d.get(multiKey , {})
+    self.d[multiKey] = multiDict
+
+    """if the modifier is set for the first time,
+    do we copy the value from the normal key or not ?"""
+    if copyInitialValue:
+      # check if the key is unset for this mode
+      if mode not in multiDict:
+        # set for first time, copy value        
+        self.set(key, self.d.get(key, defaultValue), mode=mode)
+
+    # notify watchers
+    self._notifyWatcher(key, oldValue)
+
+  def removeKeyModifier(self, key, mode=None):
+    """remove key modifier
+    NOTE: currently this just makes the key independent
+    on the current mode"""
+
+
+    # if no mode is provided, use the current one
+    if mode == None:
+        mode = self.d.get('mode', 'car')
+    if key in self.keyModifiers.keys():
+      
+      # just remove the key modifier preserving the alternative values
+      if mode in self.keyModifiers[key]['modes'].keys():
+        # get the previous value
+        options = self.m.get('options', None)
+        # remeber the old value, if not se use default from options
+        # if available
+        if options:
+          defaultValue = options.getKeyDefault(key, None)
+        else:
+          defaultValue = None
+        oldValue = self.get(key,defaultValue)
+        del self.keyModifiers[key]['modes'][mode]
+        # was this the last key ?
+        if len(self.keyModifiers[key]['modes']) == 0:
+          # no modes registered - unregister from modifiers
+          # TODO: handle non-mode modifiers in the future
+          del self.keyModifiers[key]
+        # notify watchers
+        self._notifyWatcher(key, oldValue)
+        # done
+        return True
+      else:
+        print("modrana: can't remove modifier that is not present")
+        print("key: %s, mode: %s" % (key, mode))
+        return False
+    else:
+      print("modRana: key %s has no modifier and thus cannot be removed" % key)
+      return False
+
+  def hasKeyModifier(self, key):
+    """return if a key has a key modifier"""
+    return key in self.keyModifiers.keys()
+
+  def hasKeyModifierInMode(self, key, mode=None):
+    """return if a key has a key modifier"""
+    if mode == None:
+      mode = self.d.get('mode', 'car')
+    if key in self.keyModifiers.keys():
+      return mode in self.keyModifiers[key]['modes'].keys()
+    else:
+      return False
+
+  def getModes(self):
+    modes = {
+      'cycle':'Cycle',
+      'walk':'Foot',
+      'car':'Car',
+      'train':'Train',
+      'bus':'Bus',
+    }
+    return modes
+
+  def getModeLabel(self, modeName):
+    "get a label for a given mode"
+    try:
+      return self.getModes()[modeName]
+    except KeyError:
+      print('modrana: mode %s does not exist and thus has no label' % modeName)
+      return None
+
   def update(self):
     for m in self.m.values():
       m.update()
@@ -368,13 +577,38 @@ class MapWidget(gtk.Widget):
 
     this method is called if posShiftAmount or posShiftDirection
     are set and also once at startup"""
+    # get the needed values
+    # NOTE: some of them might have been updated just now
     (sx,sy,sw,sh) = self.get('viewport')
+    shiftAmount = self.d.get('posShiftAmount', 0.75)
+    shiftDirection = self.d.get('posShiftDirection', "down")
+    scale = int(self.get('mapScale', 1))
+
+#    if key == 'viewport':
+#      (sx,sy,sw,sh) = newValue
+#    else:
+#      (sx,sy,sw,sh) = self.get('viewport')
+#
+#    if key == 'posShiftAmount':
+#      shiftAmount = newValue
+#    else:
+#      shiftAmount = self.d.get('posShiftAmount', 0.75)
+#
+#    if key == 'posShiftDirection':
+#      shiftDirection = newValue
+#    else:
+#      shiftDirection = self.d.get('posShiftDirection', "down")
+#
+#    if key == 'mapScale':
+#      scale = int(newValue)
+#    else:
+#      scale = int(self.get('mapScale', 1))
+
     x=0
     y=0
-    shiftAmount = self.d.get('posShiftAmount', 0.75)
-    """this value might show up as string, so we convert it to float, just to be sure"""
     floatShiftAmount = float(shiftAmount)
-    shiftDirection = self.d.get('posShiftDirection', "down")
+    """this value might show up as string, so we convert it to float, just to be sure"""
+
     if shiftDirection:
       if shiftDirection == "down":
         y =  sh * 0.5 * floatShiftAmount
@@ -388,8 +622,7 @@ class MapWidget(gtk.Widget):
       - 0,0 will be used """
     self.centerShift = (x,y)
     
-    # update the viewport expansion variable
-    scale = int(self.get('mapScale', 1))
+    # update the viewport expansion variable    
     tileSide = 256
     mapTiles = self.m.get('mapTiles')
     if mapTiles: # check the mapTiles for tile side length in pixels, if available
@@ -402,6 +635,28 @@ class MapWidget(gtk.Widget):
     add = int(math.ceil(float(diagonal)/tileSide))
     self.expandViewportTiles = add
 
+  def _modeChangedCB(self, key=None, oldMode=None, newMode=None):
+    """handle mode change in regards to key modifiers and option key watchers"""
+    # get keys that have both a key modifier and a watcher
+    keys = filter(lambda x: x in self.keyModifiers.keys(), self.watches.keys())
+    """ filter out only those keys that have a modifier for the new mode or
+    had a modifier in the previous mode
+    otherwise their value would not change and thus triggering a watch is not necessary """
+    keys = filter(
+                  lambda x: newMode in self.keyModifiers[x]['modes'].keys() or oldMode in self.keyModifiers[x]['modes'].keys(),
+                  keys )
+    for key in keys:
+      # try to get some value if the old value is not available
+      options = self.m.get('options', None)
+      # remeber the old value, if not se use default from options
+      # if available
+      if options:
+        defaultValue = options.getKeyDefault(key, None)
+      else:
+        defaultValue = None
+      oldValue = self.get(key, defaultValue)
+      # notify watchers
+      self._notifyWatcher(key, oldValue)
 
   def draw(self, cr, event):
     """ re/Draw the modrana GUI """
@@ -428,8 +683,11 @@ class MapWidget(gtk.Widget):
 
     menuName = self.d.get('menu', None)
     if menuName: # draw the menu
-      for m in self.m.values():
-        m.drawMenu(cr, menuName)
+      menus = self.m.get('menu', None)
+      if menus:
+        menus.mainDrawMenu(cr, menuName)
+      else:
+        print("modrana: error, menu module missing")
     else: # draw the map
       cr.set_source_rgb(0.2,0.2,0.2) # map background
       cr.rectangle(0,0,self.rect.width,self.rect.height)
@@ -484,6 +742,9 @@ class MapWidget(gtk.Widget):
         for m in self.m.values():
           m.drawScreenOverlay(cr)
 
+    # do the master overlay over everything
+    self.drawMasterOverlay(cr)
+
 #    if 'showRedrawTime' in self.d and self.d['showRedrawTime'] == True:
 
 #  def draw2(self, cr1):
@@ -727,20 +988,14 @@ class MapWidget(gtk.Widget):
     """update the current viewport in the global perzistent dictionary"""
     self.set('viewport', (self.rect.x, self.rect.y, self.rect.width, self.rect.height))
 
-  def _expose_cairo(self, event, cr):
-    #self.modules['projection'].setView( \
-    #  self.rect.x, 
-    #  self.rect.y, 
-    #  self.rect.width, 
-    #  self.rect.height)
-    
-    if(1): # optional set clipping in cairo
-      cr.rectangle(
-        self.rect.x,
-        self.rect.y,
-        self.rect.width,
-        self.rect.height)
-      cr.clip()
+  def _expose_cairo(self, event, cr):    
+    # set clipping in cairo
+    cr.rectangle(
+      self.rect.x,
+      self.rect.y,
+      self.rect.width,
+      self.rect.height)
+    cr.clip()
     
     self.draw(cr,event)
   def do_expose_event(self, event):
@@ -748,6 +1003,85 @@ class MapWidget(gtk.Widget):
     cr = self.window.cairo_create()
     return self._expose_cairo(event, cr)
 
+  # * MASTER OVERLAY *
+  def drawMasterOverlay(self, cr):
+    if self.notificationModule:
+      self.notificationModule.drawMasterOverlay(cr)
+
+  # * PROFILE PATH *
+  def getProfilePath(self):
+    """return the profile folder (create it if it does not exist)
+    NOTE: this function is provided here in the main class as some
+    ordinary modRana modules need to know the profile folder path before the
+    option module that normally handles it is fully initialized
+    (for example the config module might need to copy default
+    configuration files to the profile folder in its init)
+    """
+    # get the path
+    modRanaProfileFolderName = '.modrana'
+    userHomePath = os.getenv("HOME")
+    profileFolderPath = os.path.join(userHomePath, modRanaProfileFolderName)
+    # make sure it exists
+    createFolderPath(profileFolderPath)
+    # return it
+    return profileFolderPath
+
+  # * MAP LAYERS *
+  """map lyer information is important and needed by many modules during their initialization,
+  so it is handled here"""
+  def getMapLayers(self):
+    return self.mapLayers
+
+  def loadMapLayerInfo(self):
+    mapLayers = {}
+    configVariables = {
+        'label':'label',
+        'url':'tiles',
+        'max_zoom':'maxZoom',
+        'min_zoom':'minZoom',
+        'type':'type',
+        'folder_prefix':'folderPrefix',
+        'coordinates':'coordinates',
+                      }
+    mapConfigPath = os.path.join(self.getProfilePath(),'map_config.conf')
+    # check if the configuration files are there already
+    if not os.path.exists(mapConfigPath):
+      # nthing in prfile folder -> try to use the default configs
+      print("modRana: no config in profile folder, using default map layer configuration file")
+      mapConfigPath = os.path.join("data/default_configuration_files",'map_config.conf')
+      if not os.path.exists(mapConfigPath):
+        # no map layer config available
+        print("modRana: map layer configuration file not available")
+        return
+
+    def allNeededIn(needed, dict):
+      for key in needed:
+        if key in dict:
+          continue
+        else:
+          return False
+      return True  
+
+    try:
+      config = ConfigObj(mapConfigPath)
+      for layer in config:
+        if allNeededIn(configVariables.keys(), config[layer].keys()): # check if all neded keys are available
+          tempDict = {}
+          for var in configVariables:
+            tempDict[configVariables[var]] = config[layer][var]
+          tempDict['minZoom'] = int(tempDict['minZoom']) # convert strings to integers
+          tempDict['maxZoom'] = int(tempDict['maxZoom'])
+        else:
+          print "mapTiles: layer is badly defined/formated: %s" % layer
+
+
+        mapLayers[layer] = tempDict
+
+    except Exception, e:
+      print "mapTiles: loading map_config.conf failed: %s" % e
+
+    self.mapLayers = mapLayers
+
 class GuiBase:
   """Wrapper class for a GUI interface"""
   def __init__(self, device):
@@ -825,6 +1159,9 @@ class GuiBase:
     win.show_all()
     self.addTime("window finalized")
 
+    # load map layer info
+    self.mapWidget.loadMapLayerInfo()
+
     # start loading modules
     self.mapWidget.loadModules() # name of the folder with modules
 
@@ -942,6 +1279,8 @@ class GuiBase:
     else: # the press ended or a new press is in progress -> stop the timer
       return False
 
+
+
 if __name__ == "__main__":
   """
   suported devices (for now):
@@ -965,6 +1304,4 @@ if __name__ == "__main__":
     device = 'neo'
     print " no device string in first parameter, using: %s" % device
 
-  program = GuiBase(device)
-
-
+  program = GuiBase(device)
\ No newline at end of file
