@@ -23,6 +23,7 @@ import gtk
 import os
 import glob
 import fnmatch
+from color import Color
 from configobj import ConfigObj
 
 def getModule(m,d,i):
@@ -56,8 +57,50 @@ class icons(ranaModule):
     self.buttonFillColor = (1,1,1,1)
 
     self.updateThemeList()
-    
-  def loadFromFile(self,name,w=None,h=None):
+
+  def getIconByName(self, name, returnPixbuf=False):
+    """"get icon from current theme by exact name match
+        return None if no icon is found"""
+    # first check in the current theme folder
+    iconPathThemed = self.getCurrentThemePath()
+    # and then check the default folder if the theme folder
+    # doesnt contain this icon
+    iconPathDefault = self.getDefaultThemePath()
+    simplePaths = None
+    for path in (iconPathThemed, iconPathDefault):
+      simplePaths = glob.glob(path+"%s.*" % name)
+      if simplePaths:
+        break
+    if simplePaths:
+      iconPath = simplePaths[0] # just take the first one
+      # check if it is loadable
+      if gtk.gdk.pixbuf_get_file_info(iconPath):
+        pixbuf = gtk.gdk.pixbuf_new_from_file(iconPath)
+        if returnPixbuf:
+          return pixbuf
+        else:
+          w = pixbuf.get_width()
+          h = pixbuf.get_height()
+          # paint the pixbuf on an image surface
+          icon = cairo.ImageSurface(cairo.FORMAT_ARGB32,w,h)
+          ct = cairo.Context(icon)
+          ct2 = gtk.gdk.CairoContext(ct)
+          ct2.set_source_pixbuf(pixbuf, 0, 0)
+          ct2.paint()
+          # return the image surface
+          return icon
+      else:
+        # image is probably not loadable
+        print "icon: icon not loadable (by pixbuf loader): %s" % name
+        return None
+    else:
+      print "icons: no icon found with name: %s" % name
+      return None
+
+
+
+
+  def loadFromFile(self,name,w=None,h=None, scale=True):
     """load icon image to cache or draw the icon with cairo
     TODO: draw all icons through this function ?"""
 
@@ -100,8 +143,6 @@ class icons(ranaModule):
           except Exception, e:
             print "icons: icon probably corrupted: %s" % aboveTextIconPath
             print "%s" % e
-          # get the background icon
-#          compositeIcon = self.roundedRectangle(w, h, self.buttonFillColor, self.buttonOutlineColor)
           if pixbuf:
             compositeIcon = cairo.ImageSurface(cairo.FORMAT_ARGB32,w,h)
             ct = cairo.Context(compositeIcon)
@@ -117,7 +158,11 @@ class icons(ranaModule):
     # just use the classic icon
     elif simplePaths:
       iconPath = simplePaths.pop()
-      image = self.getImageSurface(iconPath, w, h)
+      if scale:
+        image = self.getImageSurface(iconPath, w, h)
+      else:
+        image = self.getImageSurface(iconPath)
+
     # no icon found
     else:
       print "icons: %s not found" % name
@@ -130,12 +175,19 @@ class icons(ranaModule):
     h = float(image.get_height())
     return (image,False)
 
-  def getImageSurface(self,path, w, h):
-    """load a image given by path to pixbuf and paint ti to image surface,
+  def getImageSurface(self,path, w=None, h=None):
+    """load a image given by path to pixbuf and paint it to an image surface,
        then return the image surface"""
     image = None
+
     try:
       pixbuf = gtk.gdk.pixbuf_new_from_file(path)
+      """
+      if width or height are not set, we thek tehm from the pixbuf
+      if both are not set, we disable scaling
+      """
+
+      (w,h,scale) = self._getValidParams(w, h, pixbuf)
 
       ''' create a new cairo surface to place the image on '''
       image = cairo.ImageSurface(0,w,h)
@@ -144,21 +196,42 @@ class icons(ranaModule):
       ''' create a GDK formatted Cairo context to the new Cairo native context '''
       ct2 = gtk.gdk.CairoContext(ct)
       ''' draw from the pixbuf to the new surface '''
-      ct2.set_source_pixbuf(pixbuf.scale_simple(w,h,gtk.gdk.INTERP_HYPER),0,0)
+      if scale:
+        pixbuf = pixbuf.scale_simple(w,h,gtk.gdk.INTERP_HYPER)
+      ct2.set_source_pixbuf(pixbuf,0,0)
       ct2.paint()
       ''' surface now contains the image in a Cairo surface '''
     except Exception, e:
+      print "** loading image to pixbuf failed"
       print "** filename: %s" % path
       print "** exception: %s" % e
+      return None
     return image
 
+  def _getValidParams(self, w, h, pixbuf):
+    """return width and height
+       just return w or h if they are set,
+       if they are None, use vaules from the given pixbuf"""
+    if w == None:
+      w = pixbuf.get_width()
+    if h == None:
+      h = pixbuf.get_height()
+    if w == None and h == None:
+      scale = False
+    else:
+      scale = True
+    return (w,h, scale)
+
   def getCustomIcon(self,parameterList,w,h):
     """
     there are five positional parameters:
-    fill collor,fill opacity, outline color, outline opacity,
+    fill color,fill opacity, outline color, outline opacity,
     outline width (default 8) and corner radius (default 22)
     to use default value, just don't fill in the positional parameter
     ( len(parameter) == 0 )
+    USAGE:
+    corner radius: default=22, 0 does right angle corners
+    
 
     EXAMPLE: generic:green;1.0;blue;0.5;10;15
     """
@@ -200,14 +273,14 @@ class icons(ranaModule):
     # parse the colors (if defined)
 
     if fillColorString:
-      fillColor = self.color("fill", (fillColorString,fillAlpha))
+      fillColor = Color("fill", (fillColorString,fillAlpha))
       fillColorRGBATupple = fillColor.getCairoColor()
     else:
       fillColorRGBATupple = self.buttonFillColor
 
 
     if outlineColorString:
-      outlineColor = self.color("outline", (outlineColorString,outlineAlpha))
+      outlineColor = Color("outline", (outlineColorString,outlineAlpha))
       outlineColorRGBATupple = outlineColor.getCairoColor()
     else:
       outlineColorRGBATupple = self.buttonOutlineColor
@@ -230,7 +303,7 @@ class icons(ranaModule):
       outlineColorRGBATupple = self.buttonOutlineColor
 
     # create the icon
-    icon = self.roundedRectangle(w, h, fillColorRGBATupple, outlineColorRGBATupple, outlineWidth=outlineWidth, radius=cornerRadius)
+    icon = self.roundedRectangle(w, h, fillColorRGBATupple, outlineColorRGBATupple, outlineWidth, cornerRadius)
     return icon
 
 
@@ -275,15 +348,15 @@ class icons(ranaModule):
       # run through possible "layers", which are separated by >
       compositedIcon = None
       needBackground = False
-      """icon secifications are separated by >,
+      """icon specifications are separated by >,
       which should be seen as a pointing arrow in this context
-      we composite from top -> down,
+      we composite top -> down,
       for example: "icon1>icon2"
       icon1 will be drawn over icon2
       """
       for currentName in reversed(name.split('>')): # we draw top down
         if currentName.split(':')[0] == 'generic':
-          if len(currentName.split(':',1)) == 1:
+          if currentName == "generic":
             # just the default cairo drawn icon
             needBackground = False
             genericIcon = self.roundedRectangle(w, h, self.buttonFillColor, self.buttonOutlineColor)
@@ -301,6 +374,77 @@ class icons(ranaModule):
             needBackground = False
             parametricIcon = self.getCustomIcon(semicolonSepList,w,h)
             compositedIcon = self.combineTwoIcons(compositedIcon, parametricIcon)
+        elif currentName.split(':')[0] == 'center':
+          """ "center" means that we have an icon which we want to center inside the button
+          there re two parameters - icon name and border width
+          EXAMPLE: center:more;0.1
+          -> icon name: more
+          -> border width: 10% of shortest icon side
+          """
+          # parse the parameters
+          semicolonSepList = currentName.split(':',1)[1].split(';')
+          iconName = semicolonSepList[0]
+
+          # try to get the icon
+          icon = self.getIconByName(iconName, returnPixbuf=True)
+          if (icon == None):
+            self.cantLoad.append(iconName)
+            # get a generic icon
+            genericIcon = self.roundedRectangle(w, h, self.buttonFillColor, self.buttonOutlineColor)
+            # submit it instead of the icon that failed to load
+            compositedIcon = self.combineTwoIcons(compositedIcon, genericIcon)
+          else:
+            w = int(w)
+            h = int(h)
+
+            needBackground = False
+            background = cairo.ImageSurface(cairo.FORMAT_ARGB32,w,h)
+            # scale to fit and properly place inside the button size
+
+            iw = icon.get_width()
+            ih = icon.get_height()
+            # get usable width and height
+            borderWidth = float(semicolonSepList[1])
+            if borderWidth >= 1 or borderWidth <= 0:
+              border = 0
+            else:
+              border = min(w,h)*borderWidth
+            uw = float(w - border*2)
+            uh = float(h - border*2)
+            
+            # division by zero defence
+            if not(iw == 0 or iw == 0 or uw == 0 or uh == 0):
+
+              if iw >= ih: # "landscape" icon
+                newH = uh
+                newW = iw*(uh/ih)
+                if newW > uw:
+                  newW = uw
+                  newH = ih*(newW/iw)
+
+              else: # "portrait" icon
+                newW = uw
+                newH = ih*(newW/iw)
+                if newH > uh:
+                  newH = uh
+                  newW = iw*(newH/ih)
+
+              # shift to center
+              dx = border+(uw - newW)/2.0
+              dy = border+(uh - newH)/2.0
+              
+              # crete contexts
+              ct = cairo.Context(background)
+              ctBack = gtk.gdk.CairoContext(ct)
+              icon = icon.scale_simple(int(newW),int(newH),gtk.gdk.INTERP_HYPER)
+              ctBack.set_source_pixbuf(icon, dx, dy)
+
+              # paint the icon on image surface
+              ctBack.paint()
+              
+              # composite the result with other layers
+              compositedIcon = self.combineTwoIcons(compositedIcon, background)
+
         else: # not a generic or parametric icon, try to load from file
           loadingResult = self.loadFromFile(currentName, w, h)
           if (loadingResult == False):
@@ -368,10 +512,13 @@ class icons(ranaModule):
     # make the outline propertional to the size of the button
     if outlineWidth == None:
       outlineWidth = min(width,height)*0.05
+    elif outlineWidth < 1.0: # add support for proportional outlines
+      outlineWidth = min(width,height)*outlineWidth
 
     x = 0
     y = 0
-    image = cairo.ImageSurface(0,int(width),int(height))
+    
+    image = cairo.ImageSurface(cairo.FORMAT_ARGB32,int(width),int(height))
     cr = cairo.Context(image)
     pi = 3.1415926535897931
 
@@ -380,16 +527,22 @@ class icons(ranaModule):
     # correcting for the line width
     # we also leave a line about one pixel wide free on all sides
 
-    x         = x+5
-    y         = y+5
-    width         = width-9
-    height        = height-9
+    x = x+5
+    y = y+5
+    width = width-9
+    height = height-9
 
     cr.new_sub_path()
-    cr.arc (x + width - radius, y + radius, radius, -90 * degrees, 0 * degrees)
-    cr.arc (x + width - radius, y + height - radius, radius, 0 * degrees, 90 * degrees)
-    cr.arc (x + radius, y + height - radius, radius, 90 * degrees, 180 * degrees)
-    cr.arc (x + radius, y + radius, radius, 180 * degrees, 270 * degrees)
+    if radius <= 0: # no round corners, just draw a box
+      cr.move_to(x, y)
+      cr.line_to(x+width, y)
+      cr.line_to(x+width, y+height)
+      cr.line_to(x, y+height)
+    else:
+      cr.arc (x + width - radius, y + radius, radius, -90 * degrees, 0 * degrees)
+      cr.arc (x + width - radius, y + height - radius, radius, 0 * degrees, 90 * degrees)
+      cr.arc (x + radius, y + height - radius, radius, 90 * degrees, 180 * degrees)
+      cr.arc (x + radius, y + radius, radius, 180 * degrees, 270 * degrees)
     cr.close_path()
 
     cr.set_source_rgba(*fillColor)
@@ -441,7 +594,7 @@ class icons(ranaModule):
           colorString = content[0]
           alpha = float(content[1])
           # create a new modRana color instance
-          newColor = self.color(key,colorStringAlphaTupple=(colorString,alpha))
+          newColor = Color(key,colorStringAlphaTupple=(colorString,alpha))
           colorObjects[key] = newColor
       return colorObjects
     else:
@@ -469,70 +622,4 @@ class icons(ranaModule):
 
   def colorsChangedCallback(self,colors):
     self.buttonOutlineColor = colors['main_outline'].getCairoColor()
-    self.buttonFillColor = colors['main_fill'].getCairoColor()
-
-  class color:
-    """an object representing a color
-       provides:
-       * hex color string
-       * cairo compatible rgba tupple
-       * a gtk.gdkColor object
-       """
-    def __init__(self,name,colorStringAlphaTupple=None,cairoColor=None):
-      self.name = name
-      self.valid = False
-      self.gtkColor = None
-      self.cairoColor=None
-      self.colorStringAlphaTupple = None
-      self.alpha = None
-      if colorStringAlphaTupple:
-        self.setColorFromColorStringAlphaTupple(colorStringAlphaTupple)
-
-    def __str__(self):
-      print "color name: %s" % self.name
-      print self.valid
-      print self.gtkColor
-      print self.cairoColor
-      print self.colorStringAlphaTupple
-      print self.alpha
-
-    def isValid(self):
-      return valid
-
-    def setColorFromColorStringAlphaTupple(self, colorStringAlphaTupple):
-      try:
-        (colorString,alpha) = colorStringAlphaTupple
-        gtkColor = gtk.gdk.color_parse(colorString)
-        gtkcolorRange = float(2**16)
-        cairoR = gtkColor.red/gtkcolorRange
-        cairoG = gtkColor.green/gtkcolorRange
-        cairoB = gtkColor.blue/gtkcolorRange
-        self.setAlpha(alpha)
-        self.setCairoColor(cairoR,cairoG,cairoB,alpha)
-        self.gtkColor = gtkColor
-        self.valid = True
-      except Exception, e:
-        print "** color string parsing failed **"
-        print "** input that coused this: %s" % colorStringAlphaTupple
-        print "** exception: %s" % e
-
-    def setCairoColor(self,r,g,b,a):
-      self.cairoColor = (r,g,b,a)
-
-    def setAlpha(self, alpha):
-      self.alpha = alpha
-
-    def getAlpha(self):
-      return self.alpha
-
-    def getGtkColor(self):
-      return self.gtkColor
-
-    def getCairoColor(self):
-      return self.cairoColor
-
-    def getColorStringAlphaTupple(self):
-      return self.colorStringAlphaTupple
-
-    def getColorString(self):
-      return self.gtkColor.to_string()
\ No newline at end of file
+    self.buttonFillColor = colors['main_fill'].getCairoColor()
\ No newline at end of file
