@@ -24,6 +24,7 @@ import gtk
 import time
 import math
 import geo
+import modrana_utils
 
 def getModule(m,d,i):
   return(menus(m,d,i))
@@ -40,16 +41,20 @@ class menus(ranaModule):
     self.fullscreen = False
     self.mainScreenCoords = {}
     self.userConfig = {}
-    self.notificationModule = None
     self.hideMapSreenButtons = False
     self.lastHideCheckTimestamp = time.time()
     self.itemMenuGrid = (None,[])
 
+    # tools menu
+    self.itemToolsMenuCache = (None, None)
+
     # colors - failsafe defaults
     self.mainTextColor = (0,0,0.3,1)
     self.centerButtonCircleColor = (0,0,1,0.45)
     self.spButtonFillTup = ("#ffec8b",1.0)
     self.spButtonOutlineTup = ("#8b814c",1.0)
+    self.spButtonHiFillTup = ("#ffec8b",1.0)
+    self.spButtonHiOutlineTup = ("yellow",1.0)
 
 
   def update(self):
@@ -134,8 +139,6 @@ class menus(ranaModule):
 
       # main buttons
 
-
-
       modeIcon = self.get('mode', 'car')
       # use thinner outline and smaller corner radius
       menuIcon = "%s>generic:;;;;5;10" % modeIcon
@@ -143,8 +146,15 @@ class menus(ranaModule):
 
       (x1,y1) = buttons['zoom_out']
       self.drawButton(cr, x1, y1, dx, dy, '', minusIcon, "mapView:zoomOut")
+
       (x1,y1) = buttons['menu']
-      self.drawButton(cr, x1, y1, dx, dy, 'menu', menuIcon, "set:menu:main")
+      self.drawButton(cr, x1, y1, dx, dy,
+      'menu',
+      menuIcon,
+      "set:menu:main",
+      timedAction=(self.modrana.msLongPress,"set:menu:modes"))
+
+
       (x1,y1) = buttons['zoom_in']
       self.drawButton(cr, x1, y1, dx, dy, '', plusIcon, "mapView:zoomIn")
 
@@ -203,10 +213,6 @@ class menus(ranaModule):
       (x1,y1) = buttons['scalebar']
       self.drawScalebar(cr, proj,x1,y1,w)
 
-    # master overlay hook - should be visible even when the mapscreen buttons are hidden
-    if self.notificationModule:
-      self.notificationModule.drawMasterOverlay(cr)
-
   def needRedraw(self):
     """conveninece function for asking for redraw"""
     self.set('needRedraw', True)
@@ -234,7 +240,18 @@ class menus(ranaModule):
     else:
       cr.move_to(x,y)
       pg.show_layout(layout)
-    
+
+  def showWrappedText(self,cr,text,x,y,widthLimit,fontsize=20):
+    pg = pangocairo.CairoContext(cr)
+    # create a layout for your drawing area
+    layout = pg.create_layout()
+    layout.set_width(int(widthLimit*pango.SCALE))
+    layout.set_wrap(pango.WRAP_WORD)
+    layout.set_font_description(pango.FontDescription("Sans Serif %d" % fontsize))
+    layout.set_markup(text)
+    cr.move_to(x,y)
+    pg.show_layout(layout)
+
   def drawText(self,cr,text,x,y,w,h,border=0, rgbaColor=None):
     """This is mainly used to draw the text on icons,
     this method uses pango, for the old show_text based one,
@@ -270,7 +287,8 @@ class menus(ranaModule):
     yOffset = (h-(lh * ratio))/2.0
     # move to position and show the text
     cr.move_to(x+xOffset,y+yOffset)
-    pg.scale(ratio,ratio)
+    if ratio: # handle ratio == 0.0
+      pg.scale(ratio,ratio)
     if rgbaColor != None:
       cr.set_source_rgba(*rgbaColor)
 
@@ -328,48 +346,117 @@ class menus(ranaModule):
         grid.append((x1+x*dx, y1+y*dy))
     self.itemMenuGrid = ((x1, y1, cols,rows,dx,dy),grid)
 
-  def drawMenu(self, cr, menuName):
+# menu drawing logic#
+
+  def mainDrawMenu(self, cr, menuName, args=None):
     """Draw menus"""
-#    print "current menu is:%s" % menuName
-    # Find the screen
-    if not self.d.has_key('viewport'):
-      return
-    (x1,y1,w,h) = self.get('viewport', None)
 
-    # Is it a list ?
-    if menuName in self.lists.keys(): # TODO: optimize this
-#      print "drawing list: %s" % menuName
-      self.lists[menuName].draw(cr) # draw the list
+    """
+    == Meaning of the menu persistant variable ==
+    "foo" - the default menu module has to do something with foo
+    "markers#" - the markers module has to draw the menu and gets "" as menu name
+    "markers#point" - the markers module has to draw the module
+                      and gets "point" as menu name
+    "markers#point#1 - the markers module has to draw the module
+                       and gets "point" as menu name and "1" (a string!) as args
+    """
+    split = menuName.split('#',2)
+    if len(split) == 1: # no module name found
+      self._drawOwnMenu(cr, menuName)
+    elif len(split) == 2:
+      (moduleName, menuName) = split
+      module = self.m.get(moduleName, None)
+      if module:
+        module.drawMenu(cr,menuName)
+      else:
+        print('menu: module %s that should handle menu drawing is missing' % moduleName)
+    elif len(split) == 3:
+      (moduleName, menuName, args) = split
+      module = self.m.get(moduleName, None)
+      if module:
+        module.drawMenu(cr,menuName, args)
+      else:
+        print('menu: module %s that should handle menu drawing is missing' % moduleName)
 
-      # then draw any possibly pending notifications
-      if self.notificationModule:
-        self.notificationModule.drawMasterOverlay(cr)
-      return
-    
+  def drawMenu(self, cr, menuName, args=None):
+    """handles list menus"""
+    if menuName == 'list':
+      try:
+        listName,index = args.split('#',1)
+        index = int(index)
+      except ValueError: # index not specified
+        listName = args
+        index = None
+
+      if listName in self.lists.keys():
+#        print "drawing list: %s" % menuName
+        self.lists[listName].draw(cr, index) # draw the list
+    elif menuName == 'listDetail':
+      listName,index = args.split('#',1)
+      index = int(index) # strin to int
+      if listName in self.lists.keys():
+        self.lists[listName].drawItemMenu(cr, index) # draw detailed menu
+    elif menuName == 'listDetailTools':
+      listName,index = args.split('#',1)
+      index = int(index)
+      self.lists[listName].drawItemToolsMenu(cr, index)
+
+  def _drawOwnMenu(self,cr, menuName):
+    """currently handles itemized menus"""
     # Find the menu
     menu = self.menus.get(menuName, None)
     if(menu == None):
-      if(0):
-        print "Menu %s doesn't exist, returning to main screen" % menuName
-        self.set('menu', None)
-        self.set('needRedraw', True)
-        
-        # draw any possibly pending notifications
-        if self.notificationModule:
-          self.notificationModule.drawMasterOverlay(cr)
+      print "Menu %s doesn't exist, returning to main screen" % menuName
+      self.set('menu', None)
+      self.set('needRedraw', True)
       return
+    else:
+      self.drawItemizedMenu(cr, menu, menuName)
+
+  def drawItemizedMenu(self, cr, menu, menuName):
+    """draw the item menu"""
+    vp = self.get('viewport', None)
+    if not vp:
+      print('menu:ERROR, no viewport found')
+      return
+    else:
+      (x1,y1,w,h) = vp
+      
+    # check if wide icons are to be used
+    wideButtons = menu['metadata'].get('wideButtons', False)
+
+    # check if an item should be highlighted
+    highlightId = menu['metadata'].get('highlightId', None)
+    if highlightId != None:
+      # get highlight colors
+      (spbHiFillColor,spbHiFillAlpha) = self.spButtonHiFillTup
+      (spbHiOutlineColor,spbHiOutlineAlpha) = self.spButtonHiOutlineTup
+      highlighIconDescription = "generic:%s;%f;%s;%f;;" % (spbHiFillColor, spbHiFillAlpha, spbHiOutlineColor, spbHiOutlineAlpha)
 
     # Decide how to layout the menu
-    if w > h:
-      cols = 4
-      rows = 3
-    elif w < h:
-      cols = 3
-      rows = 4
-    elif w == h:
-      cols = 4
-      rows = 4
-    
+
+    # use wide buttons #
+    if wideButtons:
+      if w > h: # landscape
+        cols = 3
+        rows = 5
+      elif w < h: # portrait
+        cols = 2
+        rows = 7
+      elif w == h: # square
+        cols = 2
+        rows = 4
+    # use roughly square buttons #
+    else:
+      if w > h: # landscape
+        cols = 4
+        rows = 3
+      elif w < h: # portrait
+        cols = 3
+        rows = 4
+      elif w == h: # square
+        cols = 4
+        rows = 4
     dx = w / cols
     dy = h / rows
 
@@ -424,6 +511,9 @@ class menus(ranaModule):
       type = item[3]
       if type=='simple':
         (text, icon, action, type, timedAction) = item
+        if id == highlightId:
+          # make the button highlighted
+          icon = "%s>>%s" % (highlighIconDescription, icon)
         self.drawButton(cr, x, y, dx, dy, text, icon, action, timedAction)
 #        (text, icon, action, type) = item
 #        self.drawButton(cr, x, y, dx, dy, text, icon, action)
@@ -435,17 +525,15 @@ class menus(ranaModule):
         # eq: "save every 3 s", "nice icon", "set:saveInterval:3s"
         text = item[0][index][0]
         icon = item[0][nextIndex][1]
+        if id == highlightId:
+          # make the button highlighted
+          icon = "%s>>%s" % (highlighIconDescription, icon)
         action = item[0][nextIndex][2]
 
         action+='|menu:toggle#%s#%s|set:needRedraw:True' % (menuName,id)
         self.drawButton(cr, x, y, dx, dy, text, icon, action)
       id += 1
 
-    # menu drawing is done, do the master overlay hook
-    if self.notificationModule:
-      self.notificationModule.drawMasterOverlay(cr)
-    return
-
   def register(self, menu, type, module):
     """Register a menu as being handled by some other module"""
     if(type == 'list'):
@@ -455,21 +543,78 @@ class menus(ranaModule):
 
   def initMenu(self,menu):
     """anitialize menu a menu dictionary instance to default parameters"""
-    self.menus[menu] = {'metadata':{'itemCount':0,'currentPage':0}}
-    
-  def clearMenu(self, menu, cancelButtonAction='set:menu:main'):
-    self.initMenu(menu)
-    timeout = self.modrana.msLongPress
-    self.addItem(menu,'','up', cancelButtonAction, 0, timedAction=(timeout,"set:menu:None"))
+    self.menus[menu] = self.getInitializedMenu()
 
-  def addItem(self, menu, text, icon=None, action=None, pos=None, timedAction=None):
-    if menu not in self.menus:
-      self.initMenu(menu)
-    itemCount = self.menus[menu]['metadata']['itemCount']
+  def getInitializedMenu(self):
+    """initialize the itemized menu datastructure
+    TODO: make this object oriented"""
+    return {'metadata':{'itemCount':0,'currentPage':0}}
+    
+  def clearMenu(self, menuName, cancelButtonAction='set:menu:main'):
+    """claer a local itemized menu instance and add the ecape button"""
+    self.initMenu(menuName)
+    timeout = self.modrana.msLongPress
+    item = self.generateItem('', 'up', cancelButtonAction, 'simple', timedAction=(timeout,"set:menu:None"))
+    self.addItems(menuName, [item])
+
+  def drawClearButton(self, cr, x, y, w, h, parentAction, icon='up'):
+    """draw a correct up button, including the timed jump-to-map action"""
+    timedAction=(self.modrana.msLongPress,"set:menu:None")
+    self.drawButton(cr, x, y, w, h, '', icon, parentAction, timedAction)
+
+  def getClearedMenu(self, cancelButtonAction='set:menu:main'):
+    """clear a given itemized menu instance, add the ecape button and return it"""
+    menu = self.getInitializedMenu()
+    timedAction=(self.modrana.msLongPress,"set:menu:None")
+    item = self.generateItem("", "up", cancelButtonAction, "simple", timedAction)
+    return self.addItemsToThisMenu(menu, [item])
+
+  def addItemsToThisMenu(self, menu, items=[]):
+    """add items to a given itemized menu datastructure and return it"""
+    if not menu:
+      menu = self.getInitializedMenu()
+    itemCount = menu['metadata']['itemCount']
     type = "simple"
-    """we are counting up from zero for the item indexes"""
-    self.menus[menu][itemCount] = (text, icon, action, type, timedAction)
-    self.menus[menu]['metadata']['itemCount'] = itemCount + 1
+    # add all items to the menu
+    for item in items:
+      (text, icon, action, type, timedAction) = item
+      """we are counting up from zero for the item indexes"""
+      menu[itemCount] = self.generateItem(text, icon, action, type, timedAction)
+      itemCount = itemCount + 1
+    menu['metadata']['itemCount'] = itemCount
+    return menu
+
+  def addItem(self, menuName, text, icon=None, action=None, pos=None, timedAction=None):
+    """add item to the local menu structure"""
+    item = self.generateItem(text, icon, action, "simple", timedAction)
+    self.menus[menuName] = self.addItemsToThisMenu(self.menus.get(menuName, None), [item,])
+
+  def getItem(self,menuName,id,):
+    """get a given itemized menu item"""
+    return self.menus[menuName][id]
+
+  def setItem(self,menuName,id, item):
+    """set a given itemized menu item to a given value"""
+    self.menus[menuName][id] = item
+
+  def highlightItem(self,menuName,id):
+    """highlight an item in a given itemmized menu, replacing the previous highlighted"""
+    self.menus[menuName]['metadata']['highlightId'] = id
+
+  def addItems(self, menuName, items):
+    """add multiple items to the local menu structure"""
+    self.menus[menuName] = self.addItemsToThisMenu(self.menus.get(menuName, None), items)
+
+  def addItemMenu(self, menuName, menu, wideButtons=False):
+    """store a given item menu
+    NOTE: if there already is a menu with the given key, it will be replaced"""
+    menu['metadata']['wideButtons'] = wideButtons
+    self.menus[menuName] = menu
+
+
+  def generateItem(self, text, icon, action, type='simple', timedAction=None):
+    """generate an itemized menu item"""
+    return (text, icon, action, type, timedAction)
 
   def addToggleItem(self, menu, textIconAction, index=0, pos=None, uniqueName=None):
     """
@@ -496,58 +641,178 @@ class menus(ranaModule):
     self.menus[menu][itemCount] = (textIconAction, index, uniqueName, type)
     self.menus[menu]['metadata']['itemCount'] = itemCount + 1
 
+#  def getToggleItem():
+# TODO: implement later
 
-  def addListableMenu(self, name, items, parrentAction, descFunction=None, drawFunction=None):
-    newListableMenu = self.listableMenu(name, self, items, parrentAction, descFunction, drawFunction,4)
+# Point menus #
+
+  def drawPointDetailMenu(self, cr, point, backAction, menuName, index):
+    """draw a detailed menu for a Point object"""
+    lat, lon = point.getLL()
+    z = self.get('z', 15)
+    button1 = ('on map#show', 'generic', 'mapView:recentre %f %f %d|set:menu:None' % (lat, lon, z))
+    button2 = ('Tools', 'tools', 'set:menu:menu#listDetailTools#%s#%d' % (menuName, index))
+    box = ('<b>%s</b>\n%s' % (point.getName(),point.getDescription()), '')
+
+    self.drawThreePlusOneMenu(cr, 'pointDetail', backAction, button1, button2, box, wrap=True)
+
+  def drawPointToolsMenu(self, cr, point, backAction):
+    """draw a detailed menu for a Point object"""
+
+    # check if the for this point is cached
+    if self.itemToolsMenuCache[0] == point:
+      # draw from cache
+      menu = self.itemToolsMenuCache[1]
+    else:
+      # generate menu structure and then cache & draw it
+      lat, lon = point.getLL()
+      menu = self.getClearedMenu(backAction)
+      gi = self.generateItem # for better readability
+      routing = 'md:route:route:type=pos2ll;toLat=%f;toLon=%f;show=start' % (lat,lon)
+      items = [
+      gi('here#route', 'generic', routing),
+      gi('to POI#add', 'generic', 'ms:menu:handleToolsMenuPoint:store'), # TODO implement this
+      gi('clear all#results', 'generic', 'set:menu:None') # TODO implement this
+      ]
+      # add the items to a menu
+      menu = self.addItemsToThisMenu(menu, items)
+      self.itemToolsMenuCache = (point, menu)
+
+    self.drawItemizedMenu(cr, menu, "fooName")
+
+  def addPointListMenu(self, name, parrentAction, points=None, goto='detail'):
+    if points:
+      c = modrana_utils.SimpleListContainer(points)
+    else:
+      c = modrana_utils.SimpleListContainer()
+
+    def describePointGo2Map(point, index, name):
+      mainText = point.getName()
+      secText, (lat, lon) = point.getAbstract(), point.getLL()
+      z = self.get('z', 15)
+      action = 'mapView:recentre %f %f %d|set:menu:None' % (lat, lon, z)
+      return(mainText,secText,action)
+
+    def describePointGo2Detail(point, index, listName):
+      mainText = point.getName()
+      secText = getDescription()
+      action = 'set:menu:menu#listDetail#%s#%d' % (name, index)
+      return(mainText,secText,action)
+
+    if goto == 'detail':
+      descFunction = describePointGo2Detail
+    else:
+      descFunction = describePointGo2Map
+
+    newListableMenu = self.ListableMenu(name, self, c, parrentAction, descFunction=descFunction, displayedItems=4)
+    newListableMenu.setDrawItemMenuMethod(self.drawPointDetailMenu)
+    newListableMenu.setDrawItemToolsMenuMethod(self.drawPointToolsMenu)
     self.lists[name] = newListableMenu
     return newListableMenu
 
+  def addListMenu(self, name, parrentAction, items=None, descFunction=None, drawFunction=None):
+    if items:
+      c = modrana_utils.SimpleListContainer(items)
+    else:
+      c = modrana_utils.SimpleListContainer()
+    newListableMenu = self.ListableMenu(name, self, c, parrentAction, descFunction, drawFunction,4)
+
+    self.lists[name] = newListableMenu
+    return newListableMenu
   
-  class listableMenu:
+  class ListableMenu:
     """a listable menu object"""
-    def __init__(self, name, menus, items, parrentAction, descFunction=None, drawFunction=None, displayedItems=3):
+    def __init__(self, name, menus, container, parrentAction, descFunction=None, drawFunction=None, displayedItems=3):
       """use custom item and description drawing functions, or use the default ones"""
-      self.descFunction = descFunction
+      # TODO: is this possible in the header ?
       if descFunction==None:
         self.descFunction = self.describeListItem
-      self.drawFunction = drawFunction
+      else:
+        self.descFunction = descFunction
       if drawFunction==None:
         self.drawFunction=self.drawListItem
+      else:
+        self.drawFunction = drawFunction
       self.index=0 #index of the first item in the current list view
       self.displayedItems = displayedItems
-      self.items = items
+      self.container = container
       self.parrentAction = parrentAction
       self.menus = menus
       self.name = name
+      self.drawItemMenuFunction = self.nop
+      self.drawItemToolsMenuFunction = self.nop
 
-    def describeListItem(self, item, index, maxIndex):
+    def getName(self):
+      """this name is also used as a key for this list by the menu module"""
+      return self.name
+
+    def nop(self, cr=None, item=None, backAction=None, index=None, menuName=None):
+      "a function that does nothing and acts as a callable placeholder"
+      pass
+
+    def setDrawItemMenuMethod(self, method):
+      """select a method for drawing a detail menu for an item"""
+      self.drawItemMenuFunction = method
+
+    def setDrawItemToolsMenuMethod(self, method):
+      """select a method for drawing a tools menu for an item"""
+      self.drawItemToolsMenuFunction = method
+
+    def setDrawMethod(self, method):
+      self.drawFunction = method
+
+    def setDescMethod(self, method):
+      self.descFunction = method
+
+    def setParrentAction(self, action):
+      self.parrentAction = action
+
+    def describeListItem(self, item, index=None, name=None):
       """default item description function
          -> get the needed strings for the default item drawing function"""
       (mainText, secText, action) = item
-      indexString = "%d/%d" % (index+1,maxIndex)
-      return(mainText, secText, indexString, action)
+      return(mainText, secText, action)
+
+    def drawItemMenu(self, cr, index):
+      item = self.container.getItem(index)
+      self.drawItemMenuFunction(cr, item, self._getBackToListAction(index), self.getName(), index)
+
+    def drawItemToolsMenu(self, cr, index):
+      item = self.container.getItem(index)
+      self.drawItemToolsMenuFunction(cr, item, self._getBackToListDetailAction(index))
+
+    def _getBackToListAction(self, index):
+      return "set:menu:menu#list#%s#%d" % (self.name, index)
+
+    def _getBackToListDetailAction(self, index):
+      return "set:menu:menu#listDetail#%s#%d" % (self.name, index)
 
     def drawListItem(self, cr, item, x, y, w, h, index, descFunction=None):
       """default list item drawing function"""
       if descFunction==None:
-        descFunction=self.describeListItem
+        descFunction=self.descFunction
 
       # * get the data for this button
-      (mainText, secText, indexString, action) = descFunction(item, index, len(self.items))
+      indexString = "%d/%d" % (index+1, self.container.getLength())
+
+      (mainText, secText, action) = descFunction(item, index, self.name)
       # * draw the background
       self.menus.drawButton(cr,x,y,w,h,'','generic', action)
 
       # * draw the text
-      border = 20
-
+      
       # 1st line: option name
-      self.menus.drawText(cr, mainText, x+border, y+border, w-2*border,20)
-      # 2nd line: current value
-      self.menus.drawText(cr, secText, x + 0.15 * w, y + 0.6 * h, w * 0.85 - border, 20)
-      # in corner: row number
-      self.menus.drawText(cr, indexString, x+0.85*w, y+3*border, w * 0.15 - border, 20)
+#      mainText = "AAAA"
+#      secText = "BBBB"
+#      indexString = "1/2"
 
-    def draw(self, cr):
+      self.menus.drawText(cr, mainText, x+w*0.10, y+h*0.1, w*0.80, h*0.5)
+      # 2nd line: current value
+      self.menus.drawText(cr, secText, x+w*0.10, y + 0.6 * h, w * 0.72, 0.4 * h)
+      # in corner: row number
+      self.menus.drawText(cr, indexString, x+0.85*w, y + 0.6 * h, w * 0.15, 0.2 * h)
+
+    def draw(self, cr, index=None):
       """draw the listable menu"""
       (e1,e2,e3,e4,alloc) = self.menus.threePlusOneMenuCoords()
       (x1,y1) = e1
@@ -564,7 +829,8 @@ class menus(ranaModule):
       # * scroll down
       self.menus.drawButton(cr, x3, y3, dx, dy, "", "down_list", "ml:menu:listMenu:%s;down" % self.name)
 
-      id = self.index
+      if index == None:
+        index = self.index
 
       # get the number of rows
       globalCount = self.menus.get('listableMenuRows', None) # try to get the global one
@@ -572,30 +838,54 @@ class menus(ranaModule):
         nrItems = self.displayedItems # use the default value
       else:
         nrItems = int(globalCount) # use the global one
-      visibleItems = self.items[id:(id+nrItems)]
+      visibleItems = self.container.getItemsInRange(index,(index+nrItems))
       if len(visibleItems): # there must be a nonzero amount of items to avoid a division by zero
         # compute item sizes
         itemBoxW = w-x4
         itemBoxH = h-y4
         itemW = itemBoxW
-#        itemH = itemBoxH/len(visibleItems)
         itemH = itemBoxH/nrItems
         for item in visibleItems:
-          self.drawFunction(cr,item,x4,y4,itemW,itemH,id) #draw the item
+          self.drawFunction(cr,item,x4,y4,itemW,itemH,index) #draw the item
           y4 = y4 + itemH # move the drawing window
-          id = id + 1 # increment the current index
+          index = index + 1 # increment the current index
 
     def scrollUp(self):
       if self.index>=1:
         self.index = self.index -1
         self.menus.needRedraw()
+
     def scrollDown(self):
-      if (self.index + 1) < len(self.items):
+      # TODO: handle containers with unknown length
+      if (self.index + 1) < self.container.getLength():
         self.index = self.index + 1
         self.menus.needRedraw()
+
     def reset(self):
       self.index = 0
 
+    def setOnceBackAction(self, action):
+      """replace the back button action with a given action for a single listable menu entry"""
+      oldAction = self.parrentAction # save the previous back action
+      self.parrentAction = action # replace with the given one
+      # restore by callback once the menu is left
+      self.menus.watch('menu', self._menuLeftCB, oldAction)
+
+    def _menuLeftCB(self,key,old,new, oldAction):
+      # restore the original back action
+      self.parrentAction = oldAction
+      # remove the watch by returning False
+      return False
+
+    def getItem(self, index):
+      return self.container.getItem(index)
+
+  def getList(self, listName):
+    """get a list by name, return None if no list is found"""
+
+  def getListItem(self, listName, index):
+    """get an item object for a given list"""
+    return self.getList(listName).getItem(index)
 
   def setupProfile(self):
     self.clearMenu('data2', "set:menu:main")
@@ -611,27 +901,18 @@ class menus(ranaModule):
         name,
         'set:layer:'+name+'|set:menu:None')
     
-  def setupTransportMenu(self):
+  def setupModesMenu(self):
     """Create menus for routing modes"""
-    self.clearMenu('transport')
-    for(label, mode) in { \
-      'Bike':'cycle',
-      'Walk':'foot',
-#      'MTB':'cycle',
-      'Car':'car',
-#      'Hike':'foot',
-#      'FastBike':'cycle',
-      'Train':'train',
-      'Bus':'bus',
-
-#      'Train':'train',
-#      'HGV':'hgv'
-      }.items():
+    self.clearMenu('modes')
+    modes = self.modrana.getModes().items()
+    modes.sort()
+    for(mode, label) in modes:
       self.addItem(
-        'transport',                       # menu
-        label,                             # label
-        mode,                     # icon
-        'set:mode:'+mode+"|set:menu:None") # action
+        'modes',  # menu
+        label,    # label
+        mode,     # icon
+        'set:mode:%s|set:menu:None' % mode, # action
+        )
 
   def setupSearchWhereMenu(self):
     self.clearMenu('searchWhere')
@@ -650,6 +931,7 @@ class menus(ranaModule):
   def setupSearchWhatMenu(self):
     self.clearMenu('searchWhat')
     self.addItem('searchWhat', 'online#address', 'generic', 'search:searchAddress')
+    self.addItem('searchWhat', 'online#wikipedia', 'generic', 'search:searchWikipedia')
     self.addItem('searchWhat', 'online#presets', 'generic', 'set:menu:searchWhere')
 
   def setupSearchMenus(self):
@@ -673,12 +955,12 @@ class menus(ranaModule):
     
   def setupPoiMenu(self):
     self.clearMenu('poi', "set:menu:main")
-    self.addItem('poi', 'POI#list', 'generic', "showPOI:setupCategoryList|set:menu:POICategories")
+    self.addItem('poi', 'POI#list', 'generic', "showPOI:setupCategoryList|set:menu:menu#list#POICategories")
     self.addItem('poi', 'POI#add new', 'generic', "set:menu:POIAddFromWhere")
     POISelectedAction1 = "showPOI:centerOnActivePOI"
-    self.addItem('poi', 'POI#go to', 'generic', "ml:showPOI:setupCategoryList:%s|set:menu:POICategories" % POISelectedAction1)
+    self.addItem('poi', 'POI#go to', 'generic', "ml:showPOI:setupCategoryList:%s|set:menu:menu#list#POICategories" % POISelectedAction1)
     POISelectedAction2 = "showPOI:routeToActivePOI"
-    self.addItem('poi', 'POI#route to', 'generic', "ml:showPOI:setupCategoryList:%s|set:menu:POICategories" % POISelectedAction2)
+    self.addItem('poi', 'POI#route to', 'generic', "ml:showPOI:setupCategoryList:%s|set:menu:menu#list#POICategories" % POISelectedAction2)
     self.addItem('poi', 'search#online', 'generic', "set:menu:searchWhere")
     self.addItem('poi', 'visible#clear', 'generic', "showPOI:clearVisiblePOI|set:menu:None")
 
@@ -691,11 +973,9 @@ class menus(ranaModule):
     self.addItem('POIAddFromWhere', 'map#from', 'generic', "ms:showPOI:storePOI:fromMap")
     self.addItem('POIAddFromWhere', 'position#current', 'generic', "ms:showPOI:storePOI:currentPosition")
 
-
-
   def setupEditBatchMenu(self):
     """this is a menu for editing settings of a batch before running the said batch"""
-    self.clearMenu('editBatch', "mapData:refreshTilecount|set:menu:batchTileDl")
+    self.clearMenu('editBatch', "mapData:refreshTilecount|set:menu:mapData#batchTileDl")
     # on exit from the editation menu refresh the tilecount
 
     # we show the values of the settings
@@ -712,7 +992,6 @@ class menus(ranaModule):
 
     layer = self.get('layer', None)
     maplayers = self.modrana.getMapLayers()
-    print maplayers[layer]['maxZoom']
     if maplayers == {}:
       maxZoomLimit == 17
     else:
@@ -735,10 +1014,10 @@ class menus(ranaModule):
     self.setupZoomDownMenu('editBatch|menu:setupEditBatchMenu', 'editBatch')
     self.setupZoomUpMenu('editBatch|menu:setupEditBatchMenu', 'editBatch')
 
-  def setupZoomUpMenu(self, nextMenu='batchTileDl', prevMenu='data'):
+  def setupZoomUpMenu(self, nextMenu='mapData#batchTileDl', prevMenu='data'):
     """in this menu, we set the maximal zoom level UP from the current zoomlevel (eq less detail)"""
     self.clearMenu('zoomUp', "set:menu:%s" % prevMenu)
-    if nextMenu == 'batchTileDl':
+    if nextMenu == 'mapData#batchTileDl':
       """if the next menu is the batch tile download menu (eq we are not called from the edit menu)
       we also send a message to refresh the tilecount after pressing the button
       (the edit menu sends the refresh message on exit so it would be redundant)"""
@@ -787,21 +1066,21 @@ class menus(ranaModule):
     self.addItem('data', 'Around view', 'generic', 'set:downloadType:data|set:downloadArea:view|set:menu:%s' % nextMenu)
     self.setupDataSubMenu()
     if self.get("batchMenuEntered", None) == True:
-      self.addItem('data', 'back to dl', 'generic', 'set:menu:batchTileDl')
+      self.addItem('data', 'back to dl', 'generic', 'set:menu:mapData#batchTileDl')
 
   def setupRouteMenu(self):
      self.clearMenu('route')
      self.addItem('route', 'Point to Point', 'generic', 'set:menu:None|route:selectTwoPoints')
      self.addItem('route', 'Here to Point#Point to Here', 'generic', 'set:menu:None|route:selectOnePoint')
      POISelectedAction2 = "showPOI:routeToActivePOI"
-     self.addItem('route', 'Here to POI', 'generic', "ml:showPOI:setupCategoryList:%s|set:menu:POICategories" % POISelectedAction2)
-     self.addItem('route', 'to Address#Address', 'generic', 'set:menu:showAdressRoute')
+     self.addItem('route', 'Here to POI', 'generic', "ml:showPOI:setupCategoryList:%s|set:menu:menu#list#POICategories" % POISelectedAction2)
+     self.addItem('route', 'to Address#Address', 'generic', 'set:menu:route#showAdressRoute')
      self.addItem('route', 'Clear', 'generic', 'route:clear|set:menu:None')
      self.addItem('route', 'route#Current', 'generic', 'set:menu:currentRoute')
 
   def setupInfoMenu(self):
     self.clearMenu('info')
-    self.addItem('info', 'About', 'generic', 'set:menu:infoAbout')
+    self.addItem('info', 'About', 'generic', 'set:menu:info#infoAbout')
 
   def setupAboutMenu(self):
     self.clearMenu('infoAbout')
@@ -813,11 +1092,11 @@ class menus(ranaModule):
     self.addItem('main', 'search', 'search', 'set:menu:searchWhat')
     self.addItem('main', 'options', 'options', 'set:menu:options')
     self.addItem('main', 'download', 'download', 'set:menu:data')
-    self.addItem('main', 'mode', 'mode', 'set:menu:transport')
+    self.addItem('main', 'mode', 'mode', 'set:menu:modes')
     self.addItem('main', 'tracklogs', 'tracklogs', 'set:menu:tracklogManagerCathegories')
-    self.addItem('main', 'log a track', 'log', 'set:menu:tracklog')
+    self.addItem('main', 'log a track', 'log', 'set:menu:tracklog#tracklog')
     self.addItem('main', 'info', 'info', 'set:menu:info')
-    self.setupTransportMenu()
+    self.setupModesMenu()
     self.setupSearchMenus()
     self.setupSearchWhatMenu()
     self.setupSearchWhereMenu()
@@ -832,20 +1111,23 @@ class menus(ranaModule):
     self.set('editBatchMenuActive', False) # at startup, the edit batch menu is inactive
 
 
-  def drawTextToSquare(self, cr, x, y, w, h, text):
+  def drawTextToSquare(self, cr, x, y, w, h, text, wrap=False):
     """draw lines of text to a square text box, \n is used as a delimiter"""
 #    (x1,y1,w1,h1) = self.get('viewport', None)
 #    dx = w / 3
 #    dy = h / 4
-    border = 30
+    border = int(min(w/30.0,h/30.0))
     spacing = 20
-    lines = text.split('\n')
-    lineCount = len(lines)
-    lineSpace = (h-2*spacing)/lineCount
-    i = 0
-    for line in lines:
-      self.showText(cr, line, x+border, y+i*lineSpace+1*spacing, w-2*border)
-      i = i + 1
+    if wrap:
+      self.showWrappedText(cr, text, x+border, y+border, w-2*border)
+    else:
+      lines = text.split('\n')
+      lineCount = len(lines)
+      lineSpace = (h-2*spacing)/lineCount
+      i = 0
+      for line in lines:
+        self.showText(cr, line, x+border, y+i*lineSpace+1*spacing, w-2*border)
+        i = i + 1
 
   def drawThreeItemHorizontalMenu(self, cr, first, second, third):
     """draw a menu, that consists from three horizontal buttons
@@ -970,7 +1252,7 @@ class menus(ranaModule):
         # in corner: row number
         self.showText(cr, "%d/%d" % (index+1, numItems), x4+0.85*w, y + 0.42 * dy, w * 0.15 - border, 20)
 
-  def drawThreePlusOneMenu(self, cr, menuName, parentAction, button1, button2, box):
+  def drawThreePlusOneMenu(self, cr, menuName, parentAction, button1, button2, box, wrap=False):
     """draw a three plus on menu"""
     (e1,e2,e3,e4,alloc) = self.threePlusOneMenuCoords()
     (x1,y1) = e1
@@ -984,7 +1266,7 @@ class menus(ranaModule):
     (boxTextLines, boxAction) = box
 
     # * draw "escape" button
-    self.drawButton(cr, x1, y1, dx, dy, "", "up", "%s:reset|%s" % (menuName, parentAction))
+    self.drawClearButton(cr, x1, y1, dx, dy, "%s:reset|%s" % (menuName, parentAction))
     # * draw the first button
     self.drawButton(cr, x2, y2, dx, dy, text1, icon1, action1)
     # * draw the second button
@@ -995,9 +1277,9 @@ class menus(ranaModule):
     self.drawButton(cr, x4, y4, w4, h4, "", "generic", boxAction)
     # * draw text to the box
     text = boxTextLines
-    self.drawTextToSquare(cr, x4, y4, w4, h4, text) # display the text in the box
+    self.drawTextToSquare(cr, x4, y4, w4, h4, text, wrap) # display the text in the box
 
-  def drawSixPlusOneMenu(self, cr, menuName, parent, fiveButtons, box):
+  def drawSixPlusOneMenu(self, cr, menuName, parentAction, fiveButtons, box):
     """draw a three plus on menu
     + support for toggle buttons"""
     (e1,e2,e3,e4,alloc) = self.threePlusOneMenuCoords()
@@ -1014,7 +1296,7 @@ class menus(ranaModule):
     (boxTextLines, boxAction) = box
 
     # * draw "escape" button
-    self.drawButton(cr, x1, y1, dx, dy, "", "up", "%s:reset|set:menu:%s" % (menuName, parent))
+    self.drawClearButton(cr, x1, y1, dx, dy, "%s:reset|%s" % (menuName, parentAction))
     # * draw the first button
     self.drawToggleButtonOld(cr, x2, y2, dx, dy, fiveButtons[0][0],fiveButtons[0][1])
     # * draw the second button
@@ -1111,16 +1393,21 @@ class menus(ranaModule):
   def colorsChangedCallback(self,colors):
     self.mainTextColor = colors['main_text'].getCairoColor()
     self.centerButtonCircleColor = colors['center_button_circle'].getCairoColor()
-    sbFill = colors['special_button__fill']
+    # normal
+    sbFill = colors['special_button_fill']
     self.spButtonFillTup = (sbFill.getColorString(),sbFill.getAlpha())
-    sbOutline = colors['special_button__outline']
+    sbOutline = colors['special_button_outline']
     self.spButtonOutlineTup = (sbOutline.getColorString(),sbOutline.getAlpha())
+    # highlight
+    sbFill = colors['main_highlight_fill']
+    self.spButtonHiFillTup = (sbFill.getColorString(),sbFill.getAlpha())
+    sbOutline = colors['main_highlight_outline']
+    self.spButtonHiOutlineTup = (sbOutline.getColorString(),sbOutline.getAlpha())
 
   def firstTime(self):
     self.set("menu",None)
     self.userConfig = self.m.get('config', None).userConfig
     # get the notification module (to implement the master overlay)
-    self.notificationModule = self.m.get('notification', None)
 
     icons = self.m.get('icons', None)
     if icons:
@@ -1156,6 +1443,20 @@ class menus(ranaModule):
       self.lastActivity = int(time.time())
       self.hideMapSreenButtons = False # show the buttons at once
       self.set('needRedraw', True)
+    elif(type=='ml' and message=='highlightItem'):
+      menuName, id = args
+      id = int(id)
+      self.highlightItem(menuName, id)
+    elif (type == 'ms' and message == 'handleToolsMenuPoint'):
+      # store the currently selected point to the POI database
+      if args == 'store':
+        point = self.itemToolsMenuCache[0]
+        store = self.m.get('storePOI', None)
+        if point and store:
+          store.storePoint(point, returnToMenu=None)
+        else:
+          print("menu: can't store point, point or storePOI module missing")
+          
     elif(message == 'toggle' and len(messageList) >= 3):
       # toggle a button
       menu = messageList[1]
