@@ -235,7 +235,13 @@ class search(ranaModule):
       # start text input for an address
       entry = self.m.get('textEntry', None)
       if entry:
-        entry.entryBox(self, 'address',description='Enter and address or location description', persistentKey="lastAddressSearchInput")
+        entry.entryBox(self, 'address',description='Enter an address or location description', persistentKey="lastAddressSearchInput")
+
+    elif message == "searchWikipedia":
+      # start text input for an address
+      entry = self.m.get('textEntry', None)
+      if entry:
+        entry.entryBox(self, 'wikipedia',description='Wikipedia search query', persistentKey="lastWikipediaSearchInput")
 
     elif message == "routeToActiveResult":
       """get a route from current position to active search result
@@ -249,7 +255,7 @@ class search(ranaModule):
 
     self.set("needRedraw", True)
 
-  def drawMenu(self, cr, menuName):
+  def drawMenu(self, cr, menuName, args=None):
     if menuName == 'searchResults':
       menus = self.m.get("menu",None)
 
@@ -348,7 +354,7 @@ class search(ranaModule):
 #    longName = name = item.getTracklogName()
 #    print filter(lambda x: x.getTracklogName() == longName, loadedTracklogs)
 #    print loadedTracklogs.index(item)
-    action = "set:menu:searchResultsItem"
+    action = "set:menu:search#searchResultsItem"
     action += "|set:searchResultsItemNr:%d" % list[index][2] # here we use the ABSOLUTE index, not the relative one
 #    action += "|set:menu:"
 #    name = item.getTracklogName().split('/').pop()
@@ -437,12 +443,12 @@ class search(ranaModule):
       print "search: warning, the units module uis missing"
       distanceString = "%1.2f km" % float(distance)
     # * draw "escape" button
-    menus.drawButton(cr, x1, y1, dx, dy, "", "up", "search:reset|set:menu:searchResults")
+    menus.drawButton(cr, x1, y1, dx, dy, "", "up", "search:reset|set:menu:search#searchResults")
     # * draw "show" button
     action2 = "search:reset|set:menu:None|set:searchResultsItemNr:%d|mapView:recentre %f %f" % (index, lat, lon)
     menus.drawButton(cr, x2, y2, dx, dy, "on map#show", "generic", action2)
 #    # * draw "add POI" button
-#    menus.drawButton(cr, x3, y3, dx, dy, "add to POI", "generic", "search:reset|search:storePOI|set:menu:searchResults")
+#    menus.drawButton(cr, x3, y3, dx, dy, "add to POI", "generic", "search:reset|search:storePOI|set:menu:search#searchResults")
     # * draw "tools" button
     menus.drawButton(cr, x3, y3, dx, dy, "tools", "tools", "set:menu:searchResultTools")
     # * draw info box
@@ -515,7 +521,7 @@ class search(ranaModule):
       # draw caption with transparent background
       text = "%s" % point['titleNoFormatting'] # result caption
 
-      print point['titleNoFormatting']
+#      print point['titleNoFormatting']
 
       cr.set_font_size(20)
       extents = cr.text_extents(text) # get the text extents
@@ -530,7 +536,7 @@ class search(ranaModule):
       m = self.m.get('clickHandler', None)
       # register clickable area
       if(m != None):
-        m.registerXYWH(rx,ry-(-rh),rw,-rh, "search:reset|set:searchResultsItemNr:%d|set:menu:searchResultsItem" % index)
+        m.registerXYWH(rx,ry-(-rh),rw,-rh, "search:reset|set:searchResultsItemNr:%d|set:menu:search#searchResultsItem" % index)
       cr.fill()
       # draw the actual text
       cr.set_source_rgba(1, 1, 1, 0.95) # slightly trasparent white
@@ -566,18 +572,18 @@ class search(ranaModule):
       text = "%s" % result['titleNoFormatting'] # result caption
       cr.set_font_size(20)
       extents = cr.text_extents(text) # get the text extents
-      (w,h) = (extents[2], extents[3])
+      (w,h) = (extents[2]*1.5, extents[3]*1.5)
       border = 2
       cr.set_line_width(2)
       cr.set_source_rgba(0, 0, 1, 0.45) # trasparent blue
-      (rx,ry,rw,rh) = (x - border+12, y + border+h*0.2 + 6, w + 4*border, -(h*1.4))
+      (rx,ry,rw,rh) = (x - border+10, y + border+h*0.2, w + 4*border, -(h*1.4))
       cr.rectangle(rx,ry,rw,rh) # create the transparent background rectangle
       cr.fill()
 
       # register clickable area
       m = self.m.get('clickHandler', None)
       if(m != None):
-        m.registerXYWH(rx,ry-(-rh),rw,-rh, "search:reset|set:searchResultsItemNr:%d|set:menu:searchResultsItem" % highlightNr)
+        m.registerXYWH(rx,ry-(-rh),rw,-rh, "search:reset|set:searchResultsItemNr:%d|set:menu:search#searchResultsItem" % highlightNr)
       cr.fill()
       
       # draw the actual text
@@ -597,7 +603,6 @@ class search(ranaModule):
       self.loadFilters() # fill the search term dictionary
 
       m.clearMenu("search", 'set:menu:main')
-      #m.addItem('search', 'centre', 'centre', 'set:menu:search_')
       for category, items in self.filters.items():
         m.clearMenu("search_%s" % category, 'set:menu:search')
         m.addItem('search', category, category.lower(), 'set:menu:search_'+category)
@@ -612,7 +617,7 @@ class search(ranaModule):
       m.addItem('search', 'query#custom', 'generic', 'search:customQuery')
 
       # setup the searchResultTools submenu
-      m.clearMenu('searchResultTools', 'set:menu:searchResultsItem')
+      m.clearMenu('searchResultTools', 'set:menu:search#searchResultsItem')
       m.addItem('searchResultTools', "here#route", "generic", "search:routeToActiveResult")
       m.addItem('searchResultTools', "add to POI", "generic", "search:reset|search:storePOI")
       m.addItem('searchResultTools', "results#clear", 'generic', 'search:clearSearch|set:menu:None')
@@ -633,19 +638,15 @@ class search(ranaModule):
       online = self.m.get('onlineServices')
       textInput = result
       if online:
-        # geocode the text input
-        results = online.geocode(textInput)
-        print("geocoding done - something found")
-        if results:
-          for r in results:
-            print(r)
-          place, (lat, lon) = results[0]
-          z = self.get('z', 15)
-          self.sendMessage('mapView:recentre %f %f %d|set:menu:None|ml:notification:m:%s;5' % (lat, lon, z, place))
-        else:
-          print("geocoding done - nothing found")
-          self.sendMessage('ml:notification:m:No results found for this address.;5')
+        # geocode the text input asynchronously
+        online.geocodeAsync(textInput, self.handleSearchResult, "address2LL")
 
+    elif key == "wikipedia":
+      online = self.m.get('onlineServices')
+      textInput = result
+      if online:
+        # search wikipedia asynchronously
+        online.wikipediaSearchAsync(textInput, self.handleSearchResult, "wikipedia")
       else:
         print("search: online services module missing")
 
@@ -653,6 +654,52 @@ class search(ranaModule):
     if key == "localSearchResultGoogle":
       print "search: GLS result recieved"
       self.localSearchResults = results
-      self.set('menu', 'searchResults')
+      self.set('menu', 'search#searchResults')
     elif key == "address2LL":
-      print  "search address2LL received"
\ No newline at end of file
+      if results:
+        print("geocoding done - something found")
+        markers = self.m.get('markers', None)
+        name = 'addressResults'
+        if markers:
+          g = markers.addGroup(name, results, menu=True)
+          menu = g.getMenuInstance()
+          if len(results) == 1: # if only one result is found, center on it righ away
+            point = results[0]
+            self._jumpToPoint(point)
+          else:
+            self.sendMessage('set:menu:menu#list#%s' % name)
+          menu.setOnceBackAction('set:menu:searchWhat')
+        else: # just jump to the first result
+          point = results[0]
+          self._jumpToPoint(point)
+      else:
+        print("geocoding done - nothing found")
+        self.sendMessage('ml:notification:m:No results found for this address.;5')
+    elif key == "wikipedia":
+      if results:
+        print("wikipedia search done - something found")
+        name = 'wikipediaResults'
+        markers = self.m.get('markers', None)
+        menu = None
+        if markers:
+          g = markers.addGroup(name, results, menu=True)
+          menu = g.getMenuInstance()
+          if len(results) == 1: # if only one result is found, center on it righ away
+            point = results[0]
+            self._jumpToPoint(point)
+          else:
+            self.sendMessage('set:menu:menu#list#%s' % name)
+          menu.setOnceBackAction('set:menu:searchWhat')
+        else: # just jump to the first result
+          point = results[0]
+          self._jumpToPoint(point)
+      else:
+        print("wikipedia search done - nothing found")
+        self.sendMessage('ml:notification:m:No results found for this query.;5')
+
+  def _jumpToPoint(self, point):
+    mw = self.m.get('mapView', None)
+    if mw:
+      mw.jump2point(point)
+
+
