@@ -49,12 +49,10 @@ class showPOI(ranaModule):
     if clickHandler:
       clickHandler.registerXYWH(x, y, x+w, y+h, 'showPOI:stopExpecting|ms:showPOI:storePOI:fromMapDone')
 
-  def drawMenu(self, cr, menuName):
+  def drawMenu(self, cr, menuName, args=None):
     """make the POI Object draw the menu :D"""
     if menuName=='POIDetail':
       self.activePOI.drawMenu(cr)
-    elif menuName=='POIDetailTools':
-      self.activePOI.drawToolsMenu(cr)
 
   def drawMapOverlay(self, cr):
     if self.drawActivePOI:
@@ -109,7 +107,10 @@ class showPOI(ranaModule):
           click = self.m.get('clickHandler', None)
           if click:
             """ make the POI caption clickable"""
-            click.registerXYWH(rx,ry-(-rh),rw,-rh, "ms:showPOI:setActivePOI:%d|set:menu:POIDetail" % id)
+            if id != None: # new POI have id == None
+              click.registerXYWH(rx,ry-(-rh),rw,-rh, "ms:showPOI:setActivePOI:%d|set:menu:showPOI#POIDetail" % id)
+            else: # the last added POI is still set, no need to set the id
+              click.registerXYWH(rx,ry-(-rh),rw,-rh, "set:menu:showPOI#POIDetail")
           cr.fill()
 
           # draw the actual text
@@ -129,9 +130,9 @@ class showPOI(ranaModule):
           """this is used for executing something special instead of going to the POIDetail menu
              after a POI is selected"""
           POISelectedAction = args[0]
-          action = "ml:showPOI:setupPOIList:%s;%s|set:menu:POIList" % ("%d",POISelectedAction)
+          action = "ml:showPOI:setupPOIList:%s;%s|set:menu:menu#list#POIList" % ("%d",POISelectedAction)
         else:
-          action = "ms:showPOI:setupPOIList:%d|set:menu:POIList"
+          action = "ms:showPOI:setupPOIList:%d|set:menu:menu#list#POIList"
         usedCategories = store.getUsedCategories()
         # convert cat_id to actions
         i = 0
@@ -140,12 +141,12 @@ class showPOI(ranaModule):
           buttonAction = action % cat_id
           usedCategories[i] = (label,desc,buttonAction)
           i = i + 1
-        menus.addListableMenu('POICategories', usedCategories,"set:menu:poi")
+        menus.addListMenu('POICategories', "set:menu:poi", usedCategories)
       elif message=='setupPOIList':
         if args:
           if type=='ms':
             catId = int(args)
-            action = 'set:menu:POIDetail' # use the default action
+            action = 'set:menu:showPOI#POIDetail' # use the default action
           elif type=='ml':
             """if the message is a message list, execute a custom action instead of the default POI detail menu
                TODO: use this even for selecting the POIDetail menu ?"""
@@ -161,7 +162,7 @@ class showPOI(ranaModule):
             buttonAction = "ms:showPOI:setActivePOI:%d|%s" % (poi_id,action)
             poiFromCategory[i] = (label,subText,buttonAction)
             i = i + 1
-          menus.addListableMenu("POIList",poiFromCategory, 'set:menu:POICategories')
+          menus.addListMenu("POIList", 'set:menu:menu#list#POICategories', poiFromCategory)
       elif type=='ms' and message=='setActivePOI':
         if args:
           POIId = int(args)
@@ -232,15 +233,8 @@ class showPOI(ranaModule):
         """setup a category chooser menu"""
         if args:
           (menu,key) = args
-          cats = store.getCategories()
-          i = 0
-          for cat in cats:
-            (label,desc,cat_id) = cat
-            action = "ms:%s:%s:%d" % (menu,key,cat_id)
-            cats[i] = (label,desc,action)
-            i = i + 1
-          menus.addListableMenu('POICategoryChooser', cats,"set:menu:poi")
-
+          self._setupPOICategoryChooser(menu, key)
+        
       elif type=='ms' and message=='setCatAndCommit':
         """selecting the category is the final stage of adding a POI"""
         if args:
@@ -252,7 +246,7 @@ class showPOI(ranaModule):
           # mark list menus for regeneration
           self.listMenusDirty = True
           # go to the new POI menu
-          self.set('menu', 'POIDetail')
+          self.set('menu', 'showPOI#POIDetail')
 
       elif message=='checkMenus':
         """check if the POI menus are "dirty" and need to be regenerated"""
@@ -263,6 +257,9 @@ class showPOI(ranaModule):
             self.sendMessage('ms:showPOI:setupPOIList:%d' % catId)
           self.listMenusDirty = False
 
+      elif message=="updateToolsMenu":
+        self.activePOI.updateToolsMenu()
+
       elif message == 'listMenusDirty':
         """something regarding the POI changed
            ,the menus might not be up to date
@@ -275,8 +272,8 @@ class showPOI(ranaModule):
           id = self.activePOI.getId()
           name = self.activePOI.getName()
           question = "Do you really want to delete:\n%s\nfrom the POI database ?" % name
-          yesAction = "ms:storePOI:deletePOI:%d|set:menu:poi" % id
-          noAction = "set:menu:POIDetailTools"
+          yesAction = "ms:storePOI:deletePOI:%d|set:menu:menu#list#POICategories" % id
+          noAction = "showPOI:updateToolsMenu|set:menu:POIDetailTools"
           ask.setupAskYesNo(question, yesAction, noAction)
 
       elif message == 'centerOnActivePOI':
@@ -308,6 +305,22 @@ class showPOI(ranaModule):
       elif message == 'clearVisiblePOI':
         self.clearVisiblePOI()
 
+  def _setupPOICategoryChooser(self, menu, key):
+    menus = self.m.get('menu', None)
+    store = self.m.get('storePOI', None)
+    cats = store.getCategories()
+    print cats
+    i = 0
+    for cat in cats:
+      (label,desc,cat_id) = cat
+      action = "ms:%s:%s:%d" % (menu,key,cat_id)
+      cats[i] = (label, desc, action)
+      i = i + 1
+
+    print cats
+
+    menus.addListMenu('POICategoryChooser',"set:menu:poi", cats)
+
   def makePOIVisible(self, POI):
     """add a POI to the list of visible POI"""
     # check if the POI is already present
@@ -360,9 +373,9 @@ class showPOI(ranaModule):
       setup the category chooser menu,
       and make sure the POI is commited after a cetegory is chosen
       """
-      self.sendMessage('ml:showPOI:setupPOICategoryChooser:showPOI;setCatAndCommit')
-      self.set('menu', 'POICategoryChooser')
-      self.sendMessage('ml:m:notification:m:Select a category for this POI;3')
+      self._setupPOICategoryChooser('showPOI', 'setCatAndCommit')
+      self.set('menu', 'menu#list#POICategoryChooser')
+      self.sendMessage('ml:notification:m:Select a category for this POI;3')
 
     # "current position as a new POI" entry chain
     elif key=='newCurrentPositionName':
@@ -371,12 +384,12 @@ class showPOI(ranaModule):
     elif key=='newCurrentPositionDescription':
       self.activePOI.setDescription(result,commit=False)
       # setting the category is the last step
-      self.sendMessage('ml:showPOI:setupPOICategoryChooser:showPOI;setCatAndCommit')
-      self.set('menu', 'POICategoryChooser')
-      print "ADASDASDASD"
+
+      # setup the category chooser
+      self._setupPOICategoryChooser('showPOI', 'setCatAndCommit')
+      self.set('menu', 'menu#list#POICategoryChooser')
       self.sendMessage('ml:notification:m:Select a category for this POI;3')
       self.set('needRedraw', True)
-      print "ADASDASDASD"
 
 if(__name__ == "__main__"):
   a = example({}, {})
