@@ -22,6 +22,7 @@ import cPickle
 import os
 import sqlite3
 import csv
+from point import Point
 
 def getModule(m,d,i):
   return(storePOI(m,d,i))
@@ -33,6 +34,9 @@ class storePOI(ranaModule):
     ranaModule.__init__(self, m, d, i)
     self.db = None
     self.tempOnlinePOI = None # temporary slot for an uncommited POI from online search
+    # to which menu to return after the POI is stored
+    # NOTE: False => unset, None => map screen
+    self.menuNameAfterStorageComplete = False
 
   def firstTime(self):
     self.connectToDb()
@@ -211,25 +215,25 @@ class storePOI(ranaModule):
       menus = self.getMenus()
       if menus:
         button1 = ('map#show on', 'generic', 'mapView:recentre %f %f|showPOI:drawActivePOI|set:menu:None' % (self.lat,self.lon))
-        button2 = ('tools', 'tools', 'set:menu:POIDetailTools')
+        button2 = ('tools', 'tools', 'showPOI:updateToolsMenu|set:menu:POIDetailTools')
         if self.label!=None and self.lat!=None and self.lon!=None and self.description!=None:
-          text = "%s\nlat:%f lon:%f\n%s" % (self.label,self.lat,self.lon,self.description)
+          text = "<big><b>%s</b></big>\n\n%s\n\nlat: <b>%f</b> lon: <b>%f</b>" % (self.label, self.description, self.lat, self.lon)
         else:
           text = "POI is being initialized"
         box = (text ,'')
-        menus.drawThreePlusOneMenu(cr, 'POIDetail', 'showPOI:checkMenus|set:menu:POIList', button1, button2, box)
+        menus.drawThreePlusOneMenu(cr, 'POIDetail', 'showPOI:checkMenus|set:menu:menu#list#POIList', button1, button2, box, wrap=True)
     
-    def drawToolsMenu(self,cr):
+    def updateToolsMenu(self):
       # setup the tools submenu
       menus = self.getMenus()
       if menus:
-        menus.clearMenu('POIDetailTools', "set:menu:POIDetail")
+        menus.clearMenu('POIDetailTools', "set:menu:showPOI#POIDetail")
         menus.addItem('POIDetailTools', 'here#route', 'generic', 'showPOI:routeToActivePOI')
         menus.addItem('POIDetailTools', 'name#edit', 'generic', 'ms:showPOI:editActivePOI:name')
         menus.addItem('POIDetailTools', 'description#edit', 'generic', 'ms:showPOI:editActivePOI:description')
         menus.addItem('POIDetailTools', 'latitude#edit', 'generic', 'ms:showPOI:editActivePOI:lat')
         menus.addItem('POIDetailTools', 'longitude#edit', 'generic', 'ms:showPOI:editActivePOI:lon')
-        menus.addItem('POIDetailTools', 'category#change', 'generic', 'ml:showPOI:setupPOICategoryChooser:showPOI;setCatAndCommit|set:menu:POICategoryChooser')
+        menus.addItem('POIDetailTools', 'category#change', 'generic', 'ml:showPOI:setupPOICategoryChooser:showPOI;setCatAndCommit|set:menu:menu#list#POICategoryChooser')
         menus.addItem('POIDetailTools', 'position#set as', 'generic', 'showPOI:centerOnActivePOI|ml:location:setPosLatLon:%f;%f' % (self.lat,self.lon))
         """just after the point is stored and and its detail menu shows up for the first time,
         it cant be deleted from the database, beucause we dont know which index it got :D
@@ -359,7 +363,14 @@ class storePOI(ranaModule):
       catInfo = self.getCategoryForId(catId)
       catName = catInfo[1]
       POIName = self.tempOnlinePOI.getName()
-      self.set('menu', 'searchResultsItem')
+      """
+      NOTE: False means the the variable is unset, as None mens the map screen
+      """
+      if self.menuNameAfterStorageComplete == False:
+        self.set('menu', 'search#searchResultsItem')
+      else:
+        self.set('menu', self.menuNameAfterStorageComplete)
+
       self.sendMessage('ml:notification:m:%s has been saved to %s;5' % (POIName, catName))
 
     elif message == "reconnectToDb":
@@ -381,7 +392,7 @@ class storePOI(ranaModule):
       self.tempOnlinePOI.setName(result, commit=False)
       self.sendMessage('ml:notification:m:Select a category for the new POI;3')
       self.sendMessage('ml:showPOI:setupPOICategoryChooser:storePOI;setCatAndCommit')
-      self.set('menu', 'POICategoryChooser')
+      self.set('menu', 'menu#list#POICategoryChooser')
 
   def dumpToCSV(self):
     """dump the db content as a CSV file"""
@@ -419,6 +430,29 @@ class storePOI(ranaModule):
     """disconnect from the database on shutdown"""
     self.disconnectFromDb()
 
+  def storePoint(self, point, returnToMenu=False):
+    """store a givne point to the POI database"""
+    # TODO: automatic saving without asking
+    # * skip name entry
+    # * and/or skip category entry
+
+    newPOI = self.getEmptyPOI()
+    newPOI.setName(point.getName(), commit=False)
+    (lat, lon) = point.getLL()
+    newPOI.setLat(lat, commit=False)
+    newPOI.setLon(lon, commit=False)
+    newPOI.setDescription(point.getDescription())
+
+    """ temporarily store the new POI to make it
+    avalable during filling its name, description, etc."""
+    self.tempOnlinePOI = newPOI
+    self.menuNameAfterStorageComplete = returnToMenu
+
+     # start the name and description entry chain
+    entry = self.m.get('textEntry', None)
+    if entry:
+      entry.entryBox(self,'onlineResultName','POI Name', initialText=point.getName())
+
   def storeGLSResult(self, result):
     """store a Google Local Search result to file"""
     name = result['titleNoFormatting']
